﻿//----------------------------------------------------
// InsertLayout.cs (c) 2019 by Enes Siljak
//----------------------------------------------------

using System;
using System.Linq;
using System.IO;
using System.Collections.Generic;
using Autodesk.AutoCAD.Runtime;
using Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Windows;

namespace XenAlati
{
    public class InsertLayout
    {
        string layoutName;
        string fLayoutDef;
        string path;
        int countFiles = 0;

        [CommandMethod("XEN_INSERTLAYOUT")]
        public void Xen_InsertLayout()
        {
            Editor acDocEd = Application.DocumentManager.MdiActiveDocument.Editor;

            try
            {
                layoutName = null;
                fLayoutDef = null;
                path = null;
                countFiles = 0;

                using (var fldrDlg = new System.Windows.Forms.FolderBrowserDialog())
                {
                    fldrDlg.Description = "Select Folder With Files To Insert Layout";
                    fldrDlg.ShowNewFolderButton = false;
                    if (fldrDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                    {
                        path = fldrDlg.SelectedPath;
                    }
                    else return;
                }

                if (path == null)
                    return;

                path = PathAddBackslash(path);

                acDocEd.WriteMessage(string.Format("\nSelected Folder: {0}\n", path));
                
                fLayoutDef = SelectDwgFileWithLayoutDef();
                if (fLayoutDef == null)
                    return;

                layoutName = SelectLayout();
                if (layoutName == null)
                    return;

                Database acSourceDb = new Database(false, true);
                acSourceDb.ReadDwgFile(fLayoutDef, FileOpenMode.OpenForReadAndAllShare, true, "");
                using (Transaction acTransSource = acSourceDb.TransactionManager.StartTransaction())
                {
                    DBDictionary SourceLayouts = acTransSource.GetObject(acSourceDb.LayoutDictionaryId, OpenMode.ForRead) as DBDictionary;

                    if (SourceLayouts.Contains(layoutName) == true)
                    {
                        Layout acSourceLayout = SourceLayouts.GetAt(layoutName).GetObject(OpenMode.ForRead) as Layout;
                        BlockTableRecord blkBlkRecEx = acTransSource.GetObject(acSourceLayout.BlockTableRecordId, OpenMode.ForRead) as BlockTableRecord;

                        ObjectIdCollection idCol = new ObjectIdCollection();
                        foreach (ObjectId id in blkBlkRecEx)
                        {
                            idCol.Add(id);
                        }
                        
                        DirectoryInfo d = new DirectoryInfo(path);
                        FileInfo[] Files = d.GetFiles("*.dwg");
                        foreach (FileInfo file in Files)
                        {
                            if (string.Compare(fLayoutDef, file.FullName) == 0)
                                continue;

                            acDocEd.WriteMessage(string.Format("\n{0}", file.FullName));
                            System.Windows.Forms.Application.DoEvents();

                            ImportLayout(acSourceDb, acTransSource, idCol, acSourceLayout, file);
                        }
                        acDocEd.WriteMessage(string.Format("\nProcessed {0} files.", countFiles));
                    }
                    else
                    {
                        Console.WriteLine("\nLayout '" + layoutName + "' could not be imported from '" + fLayoutDef + "'.");
                    }

                    acTransSource.Abort();
                }
                acSourceDb.Dispose();
            }
            catch (System.Exception ex)
            {
                acDocEd.WriteMessage("\n" + ex.ToString());
            }
        }


        private void ImportLayout(Database acSourceDb, Transaction acTransSource, ObjectIdCollection idCol, Layout acSourceLayout, FileInfo destFile)
        {
            Editor acDocEd = Application.DocumentManager.MdiActiveDocument.Editor;

            string newLayoutName = Path.GetFileNameWithoutExtension(destFile.Name);
            using (Database acDestDb = new Database(false, true))
            {
                acDestDb.ReadDwgFile(destFile.FullName, FileOpenMode.OpenForReadAndAllShare, false, null);
                using (Transaction acTransDest = acDestDb.TransactionManager.StartTransaction())
                {
                    BlockTable blkTbl = acTransDest.GetObject(acDestDb.BlockTableId, OpenMode.ForWrite) as BlockTable;
                    using (BlockTableRecord blkBlkRec = new BlockTableRecord())
                    {
                        DBDictionary destLayouts = acTransDest.GetObject(acDestDb.LayoutDictionaryId, OpenMode.ForRead) as DBDictionary;

                        if (destLayouts.Contains(newLayoutName))
                        {
                            acDocEd.WriteMessage(" Has Layout: " + newLayoutName);
                            acTransDest.Abort();
                            acDestDb.Dispose();
                            return;
                        }

                        int layoutCount = destLayouts.Count - 1;

                        blkBlkRec.Name = "*Paper_Space" + layoutCount.ToString();
                        blkTbl.Add(blkBlkRec);
                        acTransDest.AddNewlyCreatedDBObject(blkBlkRec, true);
                        acSourceDb.WblockCloneObjects(idCol,
                                                    blkBlkRec.ObjectId,
                                                    new IdMapping(),
                                                    DuplicateRecordCloning.Ignore,
                                                    false);


                        using (Layout lay = new Layout())
                        {
                            lay.LayoutName = newLayoutName;
                            lay.AddToLayoutDictionary(acDestDb, blkBlkRec.ObjectId);
                            acTransDest.AddNewlyCreatedDBObject(lay, true);
                            lay.CopyFrom(acSourceLayout);

                            DBDictionary plSets = acTransDest.GetObject(acDestDb.PlotSettingsDictionaryId, OpenMode.ForRead) as DBDictionary;
                            
                            if (lay.PlotSettingsName != "")
                            {
                                if (plSets.Contains(lay.PlotSettingsName) == false)
                                {
                                    plSets.UpgradeOpen();
                                    using (PlotSettings plSet = new PlotSettings(lay.ModelType))
                                    {
                                        plSet.PlotSettingsName = lay.PlotSettingsName;
                                        plSet.AddToPlotSettingsDictionary(acDestDb);
                                        acTransDest.AddNewlyCreatedDBObject(plSet, true);

                                        DBDictionary plSetsEx =
                                            acTransSource.GetObject(acSourceDb.PlotSettingsDictionaryId,
                                                                OpenMode.ForRead) as DBDictionary;

                                        PlotSettings plSetEx =
                                            plSetsEx.GetAt(lay.PlotSettingsName).GetObject(
                                                            OpenMode.ForRead) as PlotSettings;

                                        plSet.CopyFrom(plSetEx);
                                    }
                                }
                            }
                        }
                    }
                    acTransDest.Commit();
                }
                acDestDb.SaveAs(destFile.FullName, DwgVersion.Current);
                acDestDb.Dispose();
            }
            countFiles++;
            acDocEd.WriteMessage(" Created Layout: " + newLayoutName);
        }

                
        string PathAddBackslash(string path)
        {
            if (path == null)
                throw new ArgumentNullException(nameof(path));

            path = path.TrimEnd();

            if (PathEndsWithDirectorySeparator())
                return path;

            return path + GetDirectorySeparatorUsedInPath();

            bool PathEndsWithDirectorySeparator()
            {
                if (path.Length == 0)
                    return false;

                char lastChar = path[path.Length - 1];
                return lastChar == Path.DirectorySeparatorChar
                    || lastChar == Path.AltDirectorySeparatorChar;
            }

            char GetDirectorySeparatorUsedInPath()
            {
                if (path.Contains(Path.AltDirectorySeparatorChar))
                    return Path.AltDirectorySeparatorChar;

                return Path.DirectorySeparatorChar;
            }
        }

        string SelectDwgFileWithLayoutDef()
        {
            OpenFileDialog ofd = new OpenFileDialog(
                "Select Dwg with layout to insert",
                path,
                "dwg",
                "Dwg With Layout To Insert",
                OpenFileDialog.OpenFileDialogFlags.ForceDefaultFolder | OpenFileDialog.OpenFileDialogFlags.DefaultIsFolder
              );
            
            System.Windows.Forms.DialogResult dr = ofd.ShowDialog();
            if (dr != System.Windows.Forms.DialogResult.OK)
                return null;

            return ofd.Filename;
        }

        string SelectLayout()
        {
            Database acExDb = new Database(false, true);
            acExDb.ReadDwgFile(fLayoutDef, FileOpenMode.OpenForReadAndAllShare, true, "");
            List<string> layouts = new List<string>();
            using (Transaction acTransEx = acExDb.TransactionManager.StartTransaction())
            {
                DBDictionary layoutsEx = acTransEx.GetObject(acExDb.LayoutDictionaryId, OpenMode.ForRead) as DBDictionary;
                foreach (DBDictionaryEntry de in layoutsEx)
                {
                    if(de.Key != "Model")
                        layouts.Add(de.Key);
                }
                acTransEx.Abort();
            }
            acExDb.Dispose();

            string result = "";
            SimpleDlg.ShowSelectOptionDialog("Select Layout", "Layouts in File: ", layouts, ref result);
            return result;
        }
    }
}



